{-----------------------------------------------------------------------------
 Unit Name: SaveCustomSetting
 Author:    Erwien Saputra

 This software and source code are distributed on an as is basis, without
 warranty of any kind, either express or implied.

 This file can be redistributed, modified if you keep this header.

 Copyright  Erwien Saputra 2005  All Rights Reserved.

 Purpose:   Logic to save a custom setting to a .reg file.

 History:
 12/26/2004 - Initial version. The logic to save custom setting to a reg file,
              the license keys are not stored to the .reg file.
 01/05/2004 - Fixed a bug with the firewall, the string constant was incorrect
              and prevent saving a correct setting.
-----------------------------------------------------------------------------}

unit ConfigPersistent;

interface

type
  //Interface that expose all the necessary information to set the form size,
  //form position, the last selected tab, and the last selected custom setting.
  IMainFormProperties = interface
  ['{A38010F2-C4D7-40CA-9B92-FAE7CC84EC4B}']
    function GetTop : integer;
    function GetLeft : integer;
    function GetWidth : integer;
    function GetHeight : integer;
    function GetSelectedTabIndex : integer;
    function GetSelectedCustomSetting : string;
    procedure SetTop (const AValue : integer);
    procedure SetLeft (const AValue : integer);
    procedure SetWidth (const AValue : integer);
    procedure SetHeight (const AValue : integer);
    procedure SetSelectedCustomSetting (const AValue : string);
    procedure SetSelectedTabIndex (const AValue : integer);

    property Top : integer read GetTop write SetTop;
    property Left : integer read GetLeft write SetLeft;
    property Width : integer read GetWidth write SetWidth;
    property Height : integer read GetHeight write SetHeight;
    property SelectedTabIndex : integer read GetSelectedTabIndex
        write SetSelectedTabIndex;
    property SelectedCustomSetting : string read GetSelectedCustomSetting
        write SetSelectedCustomSetting;
  end;

  //The interface that reads configuration from a file and save it to the form.
  IConfigReader = interface
  ['{97B6BD12-53C0-4B23-BFF7-2737F473639C}']
    procedure LoadConfigToForm (const AForm : IMainFormProperties);
  end;

  //The interface that reads IMainFormProperties and save it to a file.
  IConfigWriter = interface
  ['{1EA46B41-41AB-47B9-8610-770575147163}']
    procedure SaveConfigToFile (const AForm : IMainFormProperties);
  end;

//Factory functions.
function ConfigReader : IConfigReader; forward;
function ConfigWriter : IConfigWriter; forward;

implementation

uses
  Windows, inifiles, SysUtils, Forms;

type
  //Class that implements IConfigReader and IConfigWriter.
  TConfigPersistent = class (TInterfacedObject, IConfigReader, IConfigWriter)
  private
    FConfigName : string;
    FSection : string;
    function GetWorkAreaSize : TRect;
    procedure AdjustFormSizeAndPosition(var FormTop, FormLeft, FormHeight,
        FormWidth: integer);
  protected
    procedure LoadConfigToForm (const AForm : IMainFormProperties);
    procedure SaveConfigToFile (const AForm : IMainFormProperties);
  public
    constructor Create;
  end;

const
  TOP          = 'Top';
  LEFT         = 'Left';
  WIDTH        = 'Width';
  HEIGHT       = 'Height';
  SELECTED_TAB = 'SelectedTab';
  SELECTED_CUSTOM_SETTING = 'SelectedCustomSetting';

function ConfigReader : IConfigReader;
begin
  Result := TConfigPersistent.Create;
end;

function ConfigWriter : IConfigWriter;
begin
  Result := TConfigPersistent.Create;
end;
{ TConfigPersistent }

//Save the form position, size, selected tab, and selected item to an ini file.
//The form position and the form size is adjusted if the form or part of it is
//off screen.
procedure TConfigPersistent.SaveConfigToFile(const AForm: IMainFormProperties);
var
  ini : TIniFile;
  FormTop,
  FormLeft,
  FormHeight,
  FormWidth : integer;
begin
  if Assigned (AForm) = false then
    raise Exception.Create ('Form is not assigned.');
    
  ini := TIniFile.Create (FConfigName);
  try
    FormTop     := AForm.Top;
    FormLeft    := AForm.Left;
    FormHeight  := AForm.Height;
    FormWidth   := AForm.Width;

    AdjustFormSizeAndPosition (FormTop, FormLeft, FormHeight, FormWidth);

    ini.WriteInteger (FSection, TOP, FormTop);
    ini.WriteInteger (FSection, LEFT, FormLeft);
    ini.WriteInteger (FSection, WIDTH, FormWidth);
    ini.WriteInteger (FSection, HEIGHT, FormHeight);
    ini.WriteInteger (FSection, SELECTED_TAB, AForm.SelectedTabIndex);
    ini.WriteString (FSection, SELECTED_CUSTOM_SETTING,
                     AForm.SelectedCustomSetting);
  finally
    ini.Free;
  end;
end;

//This method reads the configuration on the ini file and assigned the adjusted
//form size and position to the form.  The size and the form requires adjustment
//just in case the ini file was modified, the desktop size was modified, taskbar
//was modified or the ini file is transferred to a different PC.
procedure TConfigPersistent.LoadConfigToForm(const AForm: IMainFormProperties);
var
  ini : TIniFile;
  FormTop,
  FormLeft,
  FormHeight,
  FormWidth : integer;
begin
  if Assigned (Aform) = false then
    raise Exception.Create ('Form is not assigned.');

  if FileExists (FConfigName) = false then
    Exit;

  ini := TIniFile.Create (FConfigName);
  try
    //Read the setting from the ini file, use the AForm size as the default.
    FormTop    := ini.ReadInteger (FSection, TOP, AForm.Top);
    FormLeft   := ini.ReadInteger (FSection, LEFT, AForm.Left);
    FormWidth  := ini.ReadInteger (FSection, WIDTH, AForm.Width);
    FormHeight := ini.ReadInteger (FSection, HEIGHT, AForm.Height);
    //Adjust the form size and position.
    AdjustFormSizeAndPosition (FormTop, FormLeft, FormHeight, FormWidth);
    //Assign the adjusted size and position to the form.
    AForm.Top    := FormTop;
    AForm.Left   := FormLeft;
    AForm.Width  := FormWidth;
    AForm.Height := FormHeight;

    AForm.SelectedTabIndex := ini.ReadInteger (FSection, SELECTED_TAB,
                                               AForm.SelectedTabIndex);
    AForm.SelectedCustomSetting := ini.ReadString (FSection,
                                                   SELECTED_CUSTOM_SETTING,
                                                   AForm.SelectedCustomSetting);
  finally
    ini.Free;
  end;
end;

constructor TConfigPersistent.Create;
begin
  inherited;
  FConfigName := ChangeFileExt (ParamStr (0), '.ini');
  FSection    := ChangeFileExt (ExtractFileName (FConfigName), '');
end;

//Adjust the form size and the position. The form left, right, height and width
// will be adjusted using the usable desktop size, not the screen size. The work
//area is the desktop area minus the task bar.  Remember that some people may
//have taskbar on different sides, and sizes. People that loves Mac UI will have
//their taskbar on the top, do not replace Desktop.Top or Desktop.Left with
//assumption that taskbar is always at the bottom.
procedure TConfigPersistent.AdjustFormSizeAndPosition(var FormTop, FormLeft,
    FormHeight, FormWidth: integer);
var
  DesktopSize: TRect;
begin
  //Get the desktop size that is not covered with taskbar.
  DesktopSize := GetWorkAreaSize;

  //If the width is wider than the work are, the saved size will be reset to
  //the width of the work area.
  if FormWidth > DesktopSize.Right then
    FormWidth := DesktopSize.Right;

  //If the height is bigger than the work area, the saved size will be reset
  //to the height of the work area.
  if FormHeight > DesktopSize.Bottom then
    FormHeight := DesktopSize.Bottom;

  //If the Top of the form is less than the top of the work area, reset it.
  if FormTop < DesktopSize.Top then
    FormTop := DesktopSize.Top
  else
    //If the top if bigger than the work area size, or part of the form is out
    //of the work area, reset the top so when the app is reloaded, the form
    //will be displayed entirely.
    if (FormTop > DesktopSize.Bottom) or
       ((FormTop + FormHeight) > DesktopSize.Bottom) then
      FormTop := DesktopSize.Bottom - FormHeight;

  //If the left is out of the work area, reset it.
  if FormLeft < DesktopSize.Left then
    FormLeft := DesktopSize.Left
  else
    //If the form is off to the right or some of the form is hidden to the
    //right, reset the left property.
    if (FormLeft > DesktopSize.Right) or
       ((FormLeft + FormWidth) > DesktopSize.Right) then
      FormLeft := DesktopSize.Right - FormWidth;
end;

function TConfigPersistent.GetWorkAreaSize: TRect;
begin
  SystemParametersInfo (SPI_GETWORKAREA, 0, @Result, 0);
end;

end.
